/*
 * Copyright (c) 2005
 *	Stefan Sperling <stsp@binarchy.net>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Stefan Sperling AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Stefan Sperling OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef GAMEBOARD_H
#define GAMEBOARD_H

#include <SDL.h>

#include "graphics_mode.h"

#define MIN_WORMS 2
#define MAX_WORMS 8

typedef unsigned int gameboard_cell;
#define NUM_CELL_STATES 10
/* 0 - 7 reserved for worms */
#define CELL_FREE 8
#define CELL_WALL 9

struct worm;

#define NO_WINNER_YET 23
#define DRAW 42

typedef struct gameboard {
	unsigned int width;        /* in cells  */
	unsigned int height;       /* in cells  */
	unsigned int cell_size;    /* in pixels */
	unsigned int border_width; /* in cells  */
	SDL_Color* colors;
	gameboard_cell *grid;
	unsigned int num_worms;
	struct worm *worms[MAX_WORMS];
	unsigned int winner;
	unsigned int num_crashed_worms;
	unsigned int game_over;
} gameboard;

struct graphics_window;

gameboard* new_gameboard(graphics_mode gm, unsigned int num_worms);
void kill_gameboard(gameboard* gb);

void set_gameboard_cell(gameboard* gb, unsigned int x, unsigned int y,
		gameboard_cell value);

gameboard_cell get_gameboard_cell(gameboard *gb, unsigned int x,
		unsigned int y);
void move_worms(gameboard *gb);
#endif /* !GAMEBOARD_H */
