
/*
 * Copyright (c) 2005
 *	Stefan Sperling <stsp@binarchy.net>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Stefan Sperling AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO MENU SHALL Stefan Sperling OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef MENU_H
#define MENU_H

#include <SDL_ttf.h>

#include "options.h"

#define MENU_FONT_FILE "./fonts/actionj.ttf"
#define MENU_TITLE_FONT_SIZE 42
#define MENU_ENTRY_FONT_SIZE 24

#define MENU_TITLE CAPTION

typedef struct menu_entry {
	SDL_Surface *text;
	SDL_Surface *shaded_text;
	SDL_Rect position;
} menu_entry;

#define NUM_MENU_ENTRIES 4
#define ENTRY_NEW_GAME 0
#define ENTRY_PLAYERS 1
#define ENTRY_RESOLUTION 2
#define ENTRY_QUIT_GAME 3

typedef struct menu {
	TTF_Font *title_font;
	TTF_Font *entry_font;
	SDL_Color font_color;
	SDL_Color shaded_font_color;
	SDL_Surface *title;
	SDL_Rect title_position;
	unsigned int active_entry;
	menu_entry *entries[NUM_MENU_ENTRIES];
} menu;

struct wurmer;

menu* new_menu();
void kill_menu(menu *m);
void draw_menu(struct wurmer *w);

#endif /* !MENU_H */
