/*
 * Copyright (c) 2005
 *	Stefan Sperling <stsp@binarchy.net>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Stefan Sperling AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Stefan Sperling OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#if 0

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "level.h"
#include "gameboard.h"

/*
 * The idea of the algorithm used to create random maze levels
 * is as follows:
 *
 * Initially, the gameboard is empty (all cells are in state CELL_FREE).
 * 
 * We start tossing "bricks" (sets of CELL_WALL cells) of
 * a certain shape and size randomly onto the gameboard, thereby
 * creating an initial set of walls. There must be a certain amount
 * of free cells surrounding each brick, so they do not touch.
 * 
 * Next, we try to turn a certain amount of single free cells
 * into walls, closing gaps between the bricks. Should a wall cell
 * make it impossible for the player to reach the exit of the maze
 * if placed on the gameboard, we do not put it on the gameboard,
 * and try another location instead.
 *
 */

/* 
 * We only use one type of brick, in all possible rotations. 
 *
 * The brick defined below looks like the popular "triangular" brick
 * known from the game Tetris: ###
 *                              #
 *
 * The top-left cell of the brick is considered (0,0)
 * */      




#if 0
static brick bricks[NUM_BRICKS] = {
	{{{CELL_WALL, CELL_WALL, CELL_WALL, CELL_WALL, CELL_WALL, CELL_WALL},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE}}},

	{{{CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_WALL, CELL_WALL, CELL_WALL, CELL_WALL, CELL_WALL, CELL_WALL}}},

	{{{CELL_WALL, CELL_WALL, CELL_WALL, CELL_WALL, CELL_WALL, CELL_WALL},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE}}},

	{{{CELL_WALL, CELL_WALL, CELL_WALL, CELL_WALL, CELL_WALL, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_FREE, CELL_WALL}}},
};
#if 0
static brick bricks[NUM_BRICKS] = {
	{{{CELL_WALL, CELL_WALL, CELL_WALL},
	  {CELL_FREE, CELL_WALL, CELL_FREE},
	  {CELL_FREE, CELL_FREE, CELL_FREE}}},

	{{{CELL_FREE, CELL_FREE, CELL_FREE},
	  {CELL_FREE, CELL_WALL, CELL_FREE},
	  {CELL_WALL, CELL_WALL, CELL_WALL}}},

	{{{CELL_WALL, CELL_FREE, CELL_FREE},
	  {CELL_WALL, CELL_WALL, CELL_FREE},
	  {CELL_WALL, CELL_FREE, CELL_FREE}}},

	{{{CELL_FREE, CELL_FREE, CELL_WALL},
	  {CELL_FREE, CELL_WALL, CELL_WALL},
	  {CELL_FREE, CELL_FREE, CELL_WALL}}}
};
#endif

static int brick_fits(gameboard *gb, int x, int y);
static void put_brick(gameboard *gb, brick *b, int x, int y);

static int seeded = 0;

/* seeds the random number generator with the current time */
static void seed() {
	time_t t = time(NULL);
	if (t == (time_t) -1) {
		perror(NULL);
		return;
	}
	srandom((unsigned long)t);
	seeded = 1;
}

#define MAX_TRIES 100
static void toss_bricks(gameboard *gb, unsigned int num_bricks) {

	if ( ! seeded)
		seed();

	unsigned int i;
	for (i = 0; i < num_bricks; i++) {

		/* choose a random brick */
		unsigned long r = random() % NUM_BRICKS;
		brick *b = &bricks[r];

		int x,y;
		unsigned int tries = 0;

		do {
			/* choose random x and y coordinates for the brick */
			x = (int)(random() % gb->width);
			y = (int)(random() % gb->height);
			tries++;
		} while ( ! brick_fits(gb, x, y) && tries <= MAX_TRIES);

		/* if we tried MAX_TRIES times, there is no point trying
		 * to squeeze even more bricks onto the board */
		if (tries >= MAX_TRIES)
			return;

		put_brick(gb, b, x, y);
	}
}
#undef MAX_TRIES

static void put_brick(gameboard *gb, brick *b, int x, int y) {
	int i,j;
	for (i = 0; i < BRICK_WIDTH; i++) {
		for (j = 0; j < BRICK_HEIGHT; j++) {
			if (x + i >= 0
				|| x + i < gb->width
				|| y + j >= 0
				|| y + j < gb->height) {

				set_gameboard_cell(gb, x + i,
						y + j, b->b[i][j]);
			}
		}
	}
}

static int brick_fits(gameboard *gb, int x, int y) {

	int i, j;

	for (i = x - MIN_BRICK_DISTANCE;
			i < x + BRICK_WIDTH + MIN_BRICK_DISTANCE * 2;
			i++) {
		for (j = y - MIN_BRICK_DISTANCE;
				j < y + BRICK_HEIGHT + MIN_BRICK_DISTANCE * 2;
				j++) {
			if (i >= 0 && i < gb->width && j >= 0 && j < gb->height)
				if ((get_gameboard_cell(gb, i, j)) != CELL_FREE)
					return 0;
		}
	}

	return 1;
}

void generate_level(gameboard *gb, level *l) {
	toss_bricks(gb, l->num_initial_bricks);
}

#endif

#endif
