/*
 * Copyright (c) 2005
 *	Stefan Sperling <stsp@binarchy.net>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Stefan Sperling AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Stefan Sperling OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef LEVEL_H
#define LEVEL_H

#include "worm.h"

#define MAX_WORMS 8

typedef struct worm_start_coordinates {
	double x;
	double y;
	int x_offset;
	int y_offset;
	worm_orientation orientation;
} worm_start_coordinates;

typedef unsigned int wall_orientation;
#define WALL_VERTICAL 	0
#define WALL_HORIZONTAL	1
#define MAX_WALLS 1024 /* should you ever bump into this, raise it */

typedef struct wall {
	double x;
	double y;
	double length; /* 0 means "1 cell"
			* 1 means "till end of gameboard" */
	wall_orientation orientation;
} wall;


#define LEVEL_NO_EXIT {-1, -1}
typedef struct level_exit {
	double x;
	double y;
} level_exit;

typedef struct level {
	unsigned int num_worms;
	worm_start_coordinates start_coordinates[MAX_WORMS];
	unsigned int num_walls;
	wall walls[MAX_WALLS];
	level_exit exit;
} level;

#define LEVEL_OK 0
#define LEVEL_BAD 1

#endif /* LEVEL_H */
