/*
 * Copyright (c) 2005
 *	Stefan Sperling <stsp@binarchy.net>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Stefan Sperling AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Stefan Sperling OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "level.h"
#include "gameboard.h"

#define NUM_LEVELS 2

static const level levels[NUM_LEVELS] = {
#include "levels/level1.h"
	{NUM_WORMS, {WORM_START_COORDINATES}, NUM_WALLS, {WALLS}, LEVEL_EXIT},
/* no warnings please */
#undef NUM_WORMS
#undef WORM_START_COORDINATES
#undef NUM_WALLS
#undef WALLS
#undef LEVEL_EXIT

#include "levels/level2.h"
	{NUM_WORMS, {WORM_START_COORDINATES}, NUM_WALLS, {WALLS}, LEVEL_EXIT},
/* no warnings please */
#undef NUM_WORMS
#undef WORM_START_COORDINATES
#undef NUM_WALLS
#undef WALLS
#undef LEVEL_EXIT

};

extern char default_worm_names[MAX_WORMS][WORM_NAME_LENGTH];
extern const SDLKey worm_keys[MAX_WORMS][2];

int draw_walls(gameboard *gb, unsigned int num_level) {
	
	
}

int draw_level(gameboard * gb, unsigned int num_level) {
	draw_walls(gb, num_level);
	place_worms(gb, num_level);
}

int place_worms(gameboard * gb, unsigned int num_level) {

	unsigned int startx, starty;
	unsigned int i;

	gb->num_worms = levels[num_level].num_worms;

#define sc(x) levels[gb->num_worms].start_coordinates[i].x
	for (i = 0; i < gb->num_worms; i++) {
		
		startx = sc(x) * gb->width + sc(x_offset);
		starty = sc(y) * gb->width + sc(y_offset);
		
		gb->worms[i] = new_worm(default_worm_names[i], startx, starty,
				sc(orientation), &(gb->colors[i]),
				worm_keys[i][0], worm_keys[i][1]);
		if (gb->worms[i] == NULL) {
			fprintf(stderr, "Could not allocate memory for worm "
					"%i\n", i);
			goto new_worm_failed;
		}
		gb->worms[i]->cells_per_step = DEFAULT_CELLS_PER_STEP;
	}
#undef sc
new_worm_failed:
	for (i = 0; i < gb->num_worms; i++) {
		if (gb->worms[i] == NULL)
			break;
		else
			kill_worm(gb->worms[i]);
	}
}

