/*
 * Copyright (c) 2005
 *	Stefan Sperling <stsp@binarchy.net>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Stefan Sperling AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Stefan Sperling OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "worm.h"

worm* new_worm(char* name, int xpos, int ypos,
		int orientation, SDL_Color *color, SDLKey leftkey,
		SDLKey rightkey) {
	worm *w = (worm*)malloc(sizeof(worm));
	if (w == NULL) {
		fprintf(stderr, "Could not allocate memory for worm %s\n",
				name);
		return NULL;
	}
	int i = 0;
	while (name[i] != '\0' || i < WORM_NAME_LENGTH) {
		w->name[i] = name[i];
		i++;
	}
	while (i < WORM_NAME_LENGTH)
		w->name[i] = '\0';

	w->xpos = xpos;
	w->ypos = ypos;
	w->orientation = orientation;
	w->color = color;
	w->has_crashed = WORM_NO_CRASH;
	w->leftkey = leftkey;
	w->rightkey = rightkey;
	w->cells_per_step = DEFAULT_CELLS_PER_STEP;
	return w;
}

void kill_worm(worm *w) {
	if (w != NULL)
		free(w);
}

static const worm_orientation orientation_map_left[4] = {
	/* WORM_LOOKING_NORTH -> */ WORM_LOOKING_WEST,
	/* WORM_LOOKING_SOUTH -> */ WORM_LOOKING_EAST,
	/* WORM_LOOKING_EAST  -> */ WORM_LOOKING_NORTH,
	/* WORM_LOOKING_WEST  -> */ WORM_LOOKING_SOUTH
};

static const worm_orientation orientation_map_right[4] = {
	/* WORM_LOOKING_NORTH -> */ WORM_LOOKING_EAST,
	/* WORM_LOOKING_SOUTH -> */ WORM_LOOKING_WEST,
	/* WORM_LOOKING_EAST  -> */ WORM_LOOKING_SOUTH,
	/* WORM_LOOKING_WEST  -> */ WORM_LOOKING_NORTH
};

void reorientate(worm *w, worm_direction direction) {
	if ( ! (direction == WORM_LOOKING_NORTH
		|| direction == WORM_LOOKING_SOUTH
		|| direction == WORM_LOOKING_EAST
		|| direction == WORM_LOOKING_WEST)) {
		fprintf(stderr, "Unknown worm direction %i\n", direction);
		return;
	}
	switch (direction) {
		case WORM_GO_LEFT:
			w->orientation = orientation_map_left[w->orientation];
			break;
		case WORM_GO_RIGHT:
			w->orientation = orientation_map_right[w->orientation];
			break;
	}
}
