/*
 * Copyright (c) 2005
 *	Stefan Sperling <stsp@binarchy.net>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Stefan Sperling AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Stefan Sperling OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef WORM_H
#define WORM_H

#include <SDL.h>

/* orientations */
typedef unsigned int worm_orientation;
#define WORM_LOOKING_NORTH 0
#define WORM_LOOKING_SOUTH 1
#define WORM_LOOKING_EAST  2
#define WORM_LOOKING_WEST  3

/* directions */
typedef unsigned int worm_direction;
#define WORM_GO_LEFT  0
#define WORM_GO_RIGHT 1

typedef unsigned int worm_crash;
#define WORM_NO_CRASH		   0  /* no crash */
#define WORM_WORM_CRASH 	   1  /* a worm crashed into another worm */
#define WORM_WALL_CRASH 	   2  /* a worm crashed into a border */
#define WORM_CRASH_UNDEFINED	0xff  /* we don't have a clue what's going on */

#define WORM_NAME_LENGTH 9 /* 8 bytes ascii text + '\0' */

#define DEFAULT_CELLS_PER_STEP 1

typedef struct worm {
	char name[WORM_NAME_LENGTH]; 
	int xpos;
	int ypos;
	worm_orientation orientation; /* WORM_LOOKING_* */
	SDL_Color* color;
	worm_crash has_crashed;
	SDLKey leftkey;
	SDLKey rightkey;
	unsigned int cells_per_step;
} worm;

worm* new_worm(char* name, int xpos, int ypos,
		int orientation, SDL_Color *color, SDLKey leftkey,
		SDLKey rightkey);
void kill_worm(worm *w);
void reorientate(worm *w, worm_direction direction);
#endif /* !WORM_H */
