/*
 * Copyright (c) 2005
 *	Stefan Sperling <stsp@binarchy.net>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Stefan Sperling AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Stefan Sperling OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef WURMER_H
#define WURMER_H

#include <SDL.h>
#include "graphics_mode.h"

#define DEFAULT_DELAY 30

/* game state */
#define GAME_RUNNING 0
#define GAME_MENU 1
#define GAME_QUIT 2
#define GAME_NEW 3


struct wurmer_options;
struct worm;
struct graphics_window;
struct gameboard;
struct level;
struct menu;

typedef struct wurmer {
	struct graphics_window *gw;
	struct gameboard *gb;
	struct menu* m;
	Uint32 delay; /* controls game speed */
	struct wurmer_options *options;
	unsigned int game_state;
} wurmer;

wurmer* new_wurmer(struct wurmer_options *o);
void kill_wurmer(wurmer* w);
int run_game(wurmer* w);
wurmer* reset_wurmer(wurmer *w);
void reset_gameboard(wurmer *w);

#endif /* !WURMER_H */
